function range(start,end)
{
	this.start=start
	this.end=end
}


function isLeapYear(y)
{
	if (y % 400 == 0) return true;
	else if (y % 100 == 0) return false;
	else if (y % 4 == 0) return true;
	else return false;
}

/* number of 1-1-1900: 15020 */
function toDate(nr)
{
	var diff = nr - 15020;
	var y = 1900, m = 0, d = 1;
	var nr_days = 365;
	while (diff > nr_days)
	{
		y++;
		diff -= nr_days;
		nr_days = isLeapYear (y) ? 366 : 365;
	}
	var months = new Array (31, isLeapYear (y) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
	while (diff > months [m])
	{
		m++;
		diff -= months [m];
	}
	d += diff;
	var res = new Date();
	res.setFullYear (y, m, d);
	return res;
}

function dateToHeader(y,m)
{
	if (m < 10)
		return (String (y) + "-0" + String (m));
	else
		return (String (y) + "-" + String (m));
}

function lifetable(archives)
{
	var mindate = -1, maxdate = -1;
	lt = document.getElementById ('lifetable');
	/* rows.length: number of archives + 1 */
	rows = lt.getElementsByTagName ('tr');
	/* compute minimum and maximum date */
	for (var i = 0; i < archives.length; i++)
	{
		var ranges = archives [i];
		for (var j = 0; j < ranges.length; j++)
		{
			if (mindate == -1) mindate = ranges [j].start;
			else mindate = Math.min(mindate, ranges [j].start);
			if (maxdate == -1) maxdate = ranges [j].end;
			else maxdate = Math.max(maxdate, ranges [j].end);
		}
	}
	/* lifetable */
	for (var i = 0; i < archives.length; i++)
	{
		var ranges = archives [i];
		var cur_range = 0;
		var busy = false;
		for (var d = mindate; d <= maxdate; d++)
		{
			if (cur_range >= ranges.length)
			{
				busy = false;
			}
			else if ((!busy) && (ranges [cur_range].start == d))
			{
				busy = true;
			} 
			else if ((busy) && (ranges [cur_range].end == d - 1))
			{
				busy = false;
				cur_range++;
			}
			var td = rows [i + 1].appendChild (document.createElement ('td'));
			td.appendChild (document.createTextNode ("\u00a0"));
			if (busy)
				td.style.backgroundColor = 'green';		
			else
				td.style.backgroundColor = 'red';
		}
	}
	/* header */
	if (rows.length > 0)
	{
		var dmin = toDate (mindate);
		var dmax = toDate (maxdate);
		if (dmin.getFullYear () == dmax.getFullYear ())
		{
			/* we only have to output one month */
			if (dmin.getMonth () == dmax.getMonth ())
			{
				var th = rows [0].appendChild (document.createElement ('th'));
				th.setAttribute ("colspan", dmax.getDate () - dmin.getDate () + 1);
				th.appendChild (document.createTextNode (dateToHeader (dmin.getFullYear (), dmin.getMonth () + 1)));
			}
			/* one year, but multiple months */
			else
			{
				var months = new Array (31, isLeapYear (dmin.getFullYear ()) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
				/* first month */
				var th = rows [0].appendChild (document.createElement ('th'));
				th.setAttribute ("colspan", months [dmin.getMonth ()] - dmin.getDate () + 1);
				th.appendChild (document.createTextNode (dateToHeader (dmin.getFullYear (), dmin.getMonth () + 1)));
				/* 2nd to penultimate months */
				for (var m = dmin.getMonth () + 1; m < dmax.getMonth (); m++)
				{
					th = rows [0].appendChild (document.createElement ('th'));
					th.setAttribute ("colspan", months [m]);
					th.appendChild (document.createTextNode (dmin.getFullYear (), m + 1));
				}
				/* last month */
				th = rows [0].appendChild (document.createElement ('th'));
				th.setAttribute ("colspan", months [dmax.getMonth ()]);
				th.appendChild (document.createTextNode (dateToHeader (dmin.getFullYear (), dmax.getMonth () + 1)));
			}
		}
		/* multiple years */
		else
		{
			var months = new Array (31, isLeapYear (dmin.getFullYear ()) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
			/* first month of first year */
			th = rows [0].appendChild (document.createElement ('th'));
			th.setAttribute ("colspan", months [dmin.getMonth ()] - dmin.getDate () + 1);
			th.appendChild (document.createTextNode (dateToHeader (dmin.getFullYear (), dmin.getMonth () + 1)));
			/* rest of first year */
			for (var m = dmin.getMonth () + 1; m < 12; m++)
			{
				th = rows [0].appendChild (document.createElement ('th'));
				th.setAttribute ("colspan", months [m]);
				th.appendChild (document.createTextNode (dateToHeader (dmin.getFullYear (), m + 1)));
			}
			/* 2nd to penultimate years */
			for (var y = dmin.getFullYear () + 1; y < dmax.getFullYear (); y++)
			{
				months = new Array (31, isLeapYear (y) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
				for (var m = 0; m < 12; m++)
				{
					th = rows [0].appendChild (document.createElement ('th'));
					th.setAttribute ("colspan", months [m]);
					th.appendChild (document.createTextNode (dateToHeader (y, m)));
				}
			}
			/* last year */
			months = new Array (31, isLeapYear (dmax.getFullYear ()) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
			/* first to penultimate months of last year */
			for (var m = 0; m < dmax.getMonth (); m++)
			{
				th = rows [0].appendChild (document.createElement ('th'));
				th.setAttribute ("colspan", months [m]);
				th.appendChild (document.createTextNode (dateToHeader (dmax.getFullYear (), m + 1)));
			}
			/* last month of last year */
			th = rows [0].appendChild (document.createElement ('th'));
			th.setAttribute ("colspan", dmax.getDate ());
			th.appendChild (document.createTextNode (dateToHeader (dmax.getFullYear (), dmax.getMonth () + 1)));
		}
	}
}

