(* Globals *)
(* The EDOS project                                                 *)
(* Copyright (C)2000-2006 Berke Durak                               *)
(* Released under the GNU Lesser General Public License version 2.1 *)

exception Not_initalized_yet;;

class empty_global =
  object(self)
    method manager : Sessions.manager = raise Not_initalized_yet
    method sessions : Sessions.session_info Users.sessions = raise Not_initalized_yet
    method users_fn : string = raise Not_initalized_yet
    method shutdown = ()
  end
;;

class global ~config ~db () =
  (*** sessions *)
  let users_fn = Configuration.get_string ~default:"users.db" config "users.database" in
  let timeout = Configuration.get_float ~default:7200.0 config "users.session_timeout" in
  let manager = new Sessions.manager ~config ~db () in 
  let sessions = Users.load ~manager ~fn:users_fn ~timeout () in
  (* ***)
  object(self)
    inherit empty_global
    method manager = manager
    method sessions = sessions
    method users_fn = users_fn
    method shutdown =
      Log.info "Saving users...";
      Users.save ~sessions ~fn:users_fn ()
  end
;;

let global = ref new empty_global;;

let init ~config ~db () = global := new global ~config ~db ();;
