(* Sessions *)
(* The EDOS project                                                 *)
(* Copyright (C)2000-2006 Berke Durak                               *)
(* Released under the GNU Lesser General Public License version 2.1 *)

open Log
open Debug
;;

type session_info = {
  si_user : Users.user;
  si_sid : Users.session_id;
  si_start : float;
  si_processor : Processor.processor;
  si_bf : Buffer_formatter.buffer_formatter;
  si_mutex : Mutex.t
};;


class manager ~config ~db () =
  let time_limit = Configuration.get_int ~default:100_000 config "queries.time_limit"
  and memory_limit = Configuration.get_int ~default:100_000 config "queries.memory_limit"
  and display_limit = Configuration.get_int ~default:5000 config "queries.display_limit"
  and formatter_limit = Configuration.get_int ~default:1000000 config "queries.formatter_limit"
  in
  object
    method open_session user s_id =
      Log.info (sf "Opening session for user %S with ID %S" user.Users.u_login s_id);
      let bf = new Buffer_formatter.buffer_formatter ~limit:formatter_limit () in
      { si_user = user; 
        si_sid = s_id;
        si_start = Unix.gettimeofday ();
        si_processor =
          new Processor.processor
                ~display_limit
                ~time_limit
                ~memory_limit
                ~fmt:bf#formatter
                ~db 
                ();
        si_bf = bf;
        si_mutex = Mutex.create () }
    method close_session s_info =
      Log.info (sf "Closing session for user %S" s_info.si_user.Users.u_login)
  end
;;

