(* Users *)
(* The EDOS project                                                 *)
(* Copyright (C)2000-2006 Berke Durak                               *)
(* Released under the GNU Lesser General Public License version 2.1 *)

type session_id = string
type user = {
  u_login : string;
  u_name : string;
  u_salt : int;
  u_md5 : string;
  u_privilege_level : int;
  mutable u_last_login : float;
  mutable u_session : session_id option;
}
class type ['a] session_manager =
  object
    method close_session : 'a -> unit
    method open_session : user -> session_id -> 'a
  end
type 'a sessions
exception Already_exists
exception Unknown_user
exception Bad_password
exception No_session
val load : manager:'a session_manager -> fn:string -> ?timeout:float -> unit -> 'a sessions
val save : fn:string -> sessions:'a sessions -> unit -> unit
val digest : salt:int -> login:string -> password:string -> string
val check_login_availability : sessions:'a sessions -> login:string -> unit -> bool
val who : sessions:'a sessions -> unit -> (string * string * bool) list
val register :
  sessions:'a sessions ->
  login:string ->
  name:string -> password:string -> privilege_level:int -> unit -> unit
val logout : sessions:'a sessions -> login:string -> unit -> unit
val login :
  sessions:'a sessions -> login:string -> password:string -> unit -> session_id
val get_session_info: sessions:'a sessions -> sid:string -> unit -> user * 'a
val garbage_collect : sessions:'a sessions -> unit -> unit
val touch : sessions:'a sessions -> user:user -> unit -> unit
