(* Test_rapids *)
(* Copyright 2005-2007 Berke DURAK, INRIA Rocquencourt and The EDOS Project. *)
(* Released under the GNU LGPL version 2. *)

open Napkin;;
open Rapids;;
open Conduit;;

module DP = Debparser;;
module PS = Package_set;;

(*** package_universe *)
let package_universe db =
  let set = ref PS.empty in
  Package_index.iter (get_package_index db)
    begin fun id _ ->
      set := PS.add id !set
    end;
  !set
;;
(* ***)
(*** dump_packages *)
let dump_packages cd db =
  let oc = cd.cd_out_channel in
  let fp = cd.cd_print in
  let px = get_package_index db in
  let pu = package_universe db in
  fp oc "Total %d packages:\n" (PS.cardinal pu);
  Package_index.iter px
    begin fun id ((unit_id, version_id, architecture_id), p) ->
      let unit_name = Unit_index.find (get_unit_index db) unit_id
      and architecture_name = Architecture_index.find (get_architecture_index db) architecture_id
      and version = Version_index.get_version (Version_index.find (get_version_index db) version_id) in
      fp oc "  %s'%s on %a\n" unit_name version (Architecture_name.scribe cd) architecture_name;
    end
;;
(* ***)

let _ =
  let db = create_database () in
  let m = Array.length Sys.argv - 1 in
  let progress = new Progress.indicator ~decimation:1000 ~label:"Loading" ~channel:stdout () in
  progress#start;
  let count = ref 0 in
  for i = 1 to m - 1 do
    let fn = Sys.argv.(i) in
    progress#set_label (Printf.sprintf "Loading %S" fn);
    let ic = open_in fn in
    let ps = DP.from_channel
      ~tick:
          begin fun () ->
            incr count;
            progress#display (!count,0);
          end
      ~warn:
          begin fun w ->
            progress#warn (Printf.sprintf "Warning: %s" w)
          end
      ic
    in
    Stream.iter
      begin fun nk ->
        let _ = add_package db nk in
        (*Printf.printf "%s %s on %s has ID %a\n"
          nk.pk_unit
          nk.pk_version
          (match nk.pk_architecture with
           | None -> "(none)"
           | Some a -> a)
          print_package_id package_id*)
        ()
      end
      ps;
    close_in ic
  done;
  (*self_test db;*)
  progress#finish;
  dump_packages stdoutcd db
;;
