(* Test_dosefill_and_rapids *)
(* Copyright 2005-2007 Berke DURAK, INRIA Rocquencourt and The EDOS Project. *)
(* Released under the GNU LGPL version 2. *)

open Rapids
open Conduit

let print_string_option oc = function
  | None -> Printf.fprintf oc "(none)"
  | Some a -> Printf.fprintf oc "%s" a
;;

let handle db =
  let _package_index = get_package_index db in
  let _version_index = get_version_index db in
  (* R.self_test db *)
  let cd = stdoutcd in
  let oc = cd.cd_out_channel in
  let fp = cd.cd_print in
  try
    while true do
      match Util.split_at ' ' (input_line stdin) with
      | ["pou";u] ->
          begin
            try
              let u_id =
                try
                  Unit_index.search (get_unit_index db) u
                with
                | Not_found as x ->
                    fp oc "Unit %s not found ROTF LOL\n%!" u;
                    raise x
              in
              let ps =
                try
                  Functions.unit_id_to_package_set db u_id
                with
                | Not_found as x ->
                    fp oc "ID to package set failed\n%!";
                    raise x
              in
              fp oc "%s -> " u;
              Package_set.iter
                begin fun p_id ->
                  fp oc "%a\n%!" (Functions.scribe_package_from_id db cd ?default_architecture:None) p_id
                end
                ps;
            with
            | Not_found -> fp oc "WTF?\n%!"
          end
      | ["unit";u] ->
          begin
            try
              let u_id = Unit_index.search (get_unit_index db) u in
              fp oc "%s -> %a\n%!" u (Unit_index.scribe_id cd) u_id
            with
            | Not_found -> fp oc "Unit %s not found ROTF LOL\n%!" u
          end
      | ["ids";u;v;a] ->
        begin
          try
            let u_id = Unit_index.search (get_unit_index db) u in
            try
              let a_id = Architecture_index.search (get_architecture_index db) a in
              try
                let v_id = Version_index.get_id (Version_index.search (get_version_index db) v) in
                fp oc "%s %s %s -> %a %a %a\n%!"
                  u v a
                  (Unit_index.scribe_id cd) u_id
                  (Version_index.scribe_id cd) v_id
                  (Architecture_index.scribe_id cd) a_id
              with
              | Not_found -> fp oc "Version %s not found ROTF LOL\n%!" v
            with
            | Not_found -> fp oc "Architecture %s not found ROTF LOL\n%!" a
          with
          | Not_found -> fp oc "Unit %s not found ROTF LOL\n%!" u
        end
      | _ -> fp oc "LOL OMG WTF?!??\n%!"
    done
  with
  | End_of_file -> ()
;;

let _ =
  let spec = Waterway.specification_of_string Sys.argv.(1) in
  Waterway.with_database spec handle
;;
