(* Waterway *)
(* Copyright 2005-2007 Berke DURAK, INRIA Rocquencourt and The EDOS Project. *)
(* Released under the GNU LGPL version 2. *)

(** Glues RAPIDS and various parser modules to provide a convenient way of loading
    a RAPIDS database. *)

(** A waterway is a way to get the liquid we are interested in.  This can be in
    a DOSEbase(TM), a file (which will be a Debian APT list or an hdlist), or a
    structured collection of such files. *)
type waterway = File of string | Directory of string | Dose of string

(** It is possible to load multiple waterways at once, provided they have the same liquid.
    This is given as a list, whose components must all have the same [liquid].  Yes, this
    could have been factored but it is not very important and simpler that way. *)
type specification = (Rapids.liquid * waterway) list

(** The textual representation of a specifications is a comma-separated list as below:
  
    (deb|dose|rpm|doserpm):filename(,(deb|dose|rpm|doserpm):filename)*

    For example:

    dose:/tmp/dose,deb:/var/lib/apt/lists/ftp.fr.debian.org_debian_dists_testing_main_binary-i386_Packages

    is a valid specification. *)

val specification_of_string : string -> specification

(** Merge a waterway into the given database. *)
val merge : Rapids.db -> specification -> unit

(** Call [with_database sl handler] where [sl]
    is a waterstream specification and [handler] is your function. *)
(* val with_database : specification -> (Rapids.db -> unit) -> unit *)
